﻿using BPA.Common.Enums.CRM;
using BPA.Component.DbClient.RepositoryModel;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.Extensions;
using BPA.SaaS.Stock.Api.DTO;
using BPA.SaaS.Stock.Api.Entity;
using BPA.SaaS.Stock.Api.IRepository.Product;
using BPA.SaaS.Stock.Api.Model;
using SqlSugar;

namespace BPA.SaaS.Stock.Api.Repository.Product;

public class ProductRepository : BaseDbRepository<StockDbSugarClient, BPA_ProductCode>, IProductRepository
{
    public ProductRepository(StockDbSugarClient dbClient) : base(dbClient)
    {
    }

    public async Task<BaseResultPage<ProductListByCodeQuery>> GetProductListBySrdId(ProductListDto dto)
    {
        var total = 0;
        var result = await DbClient
            .Queryable<BPA_Batching, BPA_ProductCode>((a, b) => new JoinQueryInfos(JoinType.Inner, a.Id == b.ProductID))
            .Where((a, b) => a.IsDeleted == false && b.IsDeleted == false)
            .Select((a, b) => new ProductListByCodeQuery
            {
                Id = a.Id,
                CodeId = b.Id,
                Code = b.Code,
                Aittribute = a.Aittribute,
                Name = a.Batching_Name,
                TypeID = a.TypeID,
                StockUint = a.StockUint,
                Specs = a.Specs,
                PackUnit = b.PackUnit,
                Proportion = b.Proportion
            })
            .ToPageListAsync(dto.Current, dto.PageSize, total);
        return BaseResultPage<ProductListByCodeQuery>.BuildOK(result, total);
    }

    public async Task<BaseResultPage<ProductListQuery>> GetProductList(ProductListQuery dto, string groupId)
    {
        var total = 0;
        var result = await DbClient
            .Queryable<BPA_Batching, BPA_ProductType, BPA_Uint>((a, b, c) => new JoinQueryInfos(JoinType.Inner, a.TypeID == b.Id, JoinType.Inner, a.StockUint == c.Id))
            .Where((a, b, c) => a.IsDeleted == false && b.IsDeleted == false && c.IsDeleted == false)
            .WhereIF(groupId.IsNotEmpty(), a => a.GroupId == groupId)
            .WhereIF(dto.Name.IsNotEmpty(), a => a.Batching_Name == dto.Name)
            .WhereIF(dto.Code.IsNotEmpty(), a => a.Code == dto.Code)
            .WhereIF(dto.StockUint.IsNotEmpty(), a => a.Id == dto.StockUint)
            .WhereIF(dto.Specs.IsNotEmpty(), a => a.Specs == dto.Specs)
            .WhereIF(dto.Aittribute.IsNotEmpty(), a => a.Aittribute.ToString() == dto.Aittribute)
            .WhereIF(dto.TypeID.IsNotEmpty(), a => a.TypeID == dto.TypeID)
            .WhereIF(dto.Status.HasValue, a => a.Status == dto.Status)
            .WhereIF(dto.CreateAt.HasValue, a => SqlFunc.DateIsSame(a.CreateAt, Convert.ToDateTime(dto.CreateAt), DateType.Day))
            .WhereIF(dto.UpdateAt.HasValue, a => SqlFunc.DateIsSame(a.UpdateAt, Convert.ToDateTime(dto.UpdateAt), DateType.Day))
            .OrderBy((a, b, c) => a.CreateAt, OrderByType.Desc)
            .Select((a, b, c) => new ProductListQuery
            {
                Id = a.Id,
                Name = a.Batching_Name,
                Code = a.Code,
                StockUint = a.StockUint,
                StockUintName = c.Name,
                Specs = a.Specs,
                Aittribute = a.Aittribute.ToString(),
                TypeID = b.Id,
                TypeName = b.Name,
                CreateAt = a.CreateAt,
                UpdateAt = a.UpdateAt,
                Status = a.Status,
                // StatusText = a.Status.GetDescribe(),
                batchingType = a.Batching_Type
            })
            .ToPageListAsync(dto.Current, dto.PageSize, total);
        return BaseResultPage<ProductListQuery>.BuildOK(result, total);
    }

    public async Task<BaseResult<bool>> AddProduct(ProductInfoDto dto)
    {
        await SimpleClient.Context.Ado.UseTranAsync(async () =>
        {
            var bpaProduct = dto.MapTo<BPA_Batching, ProductInfoDto>();
            bpaProduct.Aittribute = Convert.ToInt32(dto.Aittribute);
            bpaProduct.Batching_Name = dto.Name;
            bpaProduct.Batching_Type = dto.batchingType;
            bpaProduct.Status = EnumCommonStatus.ENABLE;
            await DbClient.Insertable(bpaProduct).ExecuteCommandAsync();

            var productCodes = new List<BPA_ProductCode>();
            if (dto.membersList != null && dto.membersList.Any())
            {
                productCodes.AddRange(dto.membersList.Select(productDetailedDto => new BPA_ProductCode
                {
                    ProductID = bpaProduct.Id,
                    PackUnit = productDetailedDto.PackUnit,
                    Code = productDetailedDto.PackCode,
                    Proportion = Convert.ToDecimal(productDetailedDto.Proportion)
                }));
                await DbClient.Insertable(productCodes).ExecuteCommandAsync();
            }
        });
        return BaseResult<bool>.BuildOK(true);
    }

    public async Task<BaseResult<bool>> UpdateProduct(ProductInfoDto dto)
    {
        await SimpleClient.Context.Ado.UseTranAsync(async () =>
        {
            var bpaProduct = dto.MapTo<BPA_Batching, ProductInfoDto>();
            bpaProduct.Aittribute = Convert.ToInt32(dto.Aittribute);
            bpaProduct.Batching_Name = dto.Name;
            bpaProduct.Batching_Type = dto.batchingType;
            await DbClient.Insertable(bpaProduct).IgnoreColumns(true).ExecuteCommandAsync();

            var productCode = await DbClient.Queryable<BPA_ProductCode>().Where(x => x.IsDeleted == false && x.ProductID == dto.Id).ToListAsync();
            if (productCode.Any())
            {
                foreach (var x in productCode)
                {
                    x.IsDeleted = true;
                    x.DeleteAt = DateTime.Now;
                    x.DeleteBy = ""; //TODO:
                }

                await DbClient.Updateable(productCode).ExecuteCommandAsync();
            }

            var productCodes = new List<BPA_ProductCode>();
            if (dto.membersList != null && dto.membersList.Any())
            {
                productCodes.AddRange(dto.membersList.Select(productDetailedDto => new BPA_ProductCode
                {
                    ProductID = bpaProduct.Id,
                    PackUnit = productDetailedDto.PackUnit,
                    Code = productDetailedDto.PackCode,
                    Proportion = Convert.ToDecimal(productDetailedDto.Proportion)
                }));
                await DbClient.Insertable(productCodes).ExecuteCommandAsync();
            }
        });
        return BaseResult<bool>.BuildOK(true);
    }

    public async Task<BaseResult<bool>> BatchDelProduct(List<string> ids)
    {
        await SimpleClient.Context.Ado.UseTranAsync(async () =>
        {
            var bpaBatching = await DbClient.Queryable<BPA_Batching>().In(ids).ToListAsync();
            bpaBatching.ForEach(x =>
            {
                x.IsDeleted = true;
                x.DeleteAt = DateTime.Now;
                x.DeleteBy = ""; //TODO
            });
            await DbClient.Updateable(bpaBatching).ExecuteCommandAsync();

            var productCode = await DbClient.Queryable<BPA_ProductCode>().Where(x => x.IsDeleted == false && ids.Contains(x.ProductID)).ToListAsync();
            if (productCode.Any())
            {
                productCode.ForEach(x =>
                {
                    x.IsDeleted = true;
                    x.DeleteAt = DateTime.Now;
                    x.DeleteBy = ""; //TODO
                });
                await DbClient.Updateable(productCode).ExecuteCommandAsync();
            }
        });
        return BaseResult<bool>.BuildOK(true);
    }

    public async Task<BaseResult<List<ProductDetailedDto>>> GetProductCode(string productId)
    {
        var result = await DbClient
            .Queryable<BPA_ProductCode>()
            .Where(x => x.IsDeleted == false && x.ProductID == productId)
            .Select(x => new ProductDetailedDto
            {
                Id = x.Id,
                PackCode = x.Code,
                PackUnit = x.PackUnit,
                Proportion = x.Proportion
            })
            .ToListAsync();
        return BaseResult<List<ProductDetailedDto>>.BuildOK(result);
    }

    public async Task<BaseResult<List<ProductInfoDto>>> IsModifyOrDel(string[] ids, string typeValue, EnumCommonStatus? status)
    {
        var itemName = string.Empty;
        var isUpdate = false;
        var itemTypeId = string.Empty;

        if (ids.Length > 0)
        {
            itemTypeId = typeValue == "1" ? " where IsDeleted=0 AND TypeID in (" : " where IsDeleted=0 AND StockUint in (";

            foreach (var t in ids)
            {
                //删除
                if (!status.HasValue)
                {
                    if (t.Split(",").Length > 0)
                    {
                        for (var j = 0; j < t.Split(",").Length; j++)
                        {
                            itemTypeId += $"'{t.Split(",")[j]}',";
                        }
                    }
                }
                else
                {
                    itemTypeId += $"'{t}',";
                }
            }

            itemTypeId = itemTypeId.Substring(0, itemTypeId.LastIndexOf(','));
            itemTypeId += ")";

            var sql = @"SELECT * FROM BPA_Batching" + itemTypeId;
            var items = new List<ProductInfoDto>();
            if (typeValue == "1")
            {
                items = await DbClient.SqlQueryable<BPA_Batching>(sql).Where(it => it.IsDeleted == false).Select(a => new ProductInfoDto
                {
                    TypeID = a.TypeID,
                    Status = a.Status
                }).GroupBy((a) => a.TypeID).GroupBy(a => a.Status).ToListAsync();
            }
            else
            {
                items = await DbClient.SqlQueryable<BPA_Batching>(sql).Where(it => it.IsDeleted == false).Select(a => new ProductInfoDto
                {
                    StockUint = a.StockUint,
                    Status = a.Status
                }).GroupBy((a) => a.StockUint).GroupBy(a => a.Status).ToListAsync();
            }

            if (status.HasValue) //修改是时判断是否可以修改状态
            {
                items.ForEach(x =>
                {
                    if (x.Status != status)
                    {
                        isUpdate = true;
                    }
                });
                if (isUpdate) //判断物料类型或者物料单位不能修改
                {
                    var itemCls = new List<BPA_ProductType>();
                    var itemUint = new List<BPA_Uint>();
                    if (items.Count > 0)
                    {
                        items.ForEach(async x =>
                        {
                            if (typeValue == "1")
                            {
                                itemCls = await DbClient.Queryable<BPA_ProductType>().Where(it => it.IsDeleted == false && it.Id == x.TypeID).ToListAsync();
                                if (itemCls.Count > 0)
                                {
                                    itemCls.ForEach(x =>
                                    {
                                        if (!itemName.Contains(x.Name))
                                        {
                                            itemName += x.Name + "、";
                                        }
                                    });
                                }

                                x.itemName = itemName;
                            }
                            else
                            {
                                itemUint = await DbClient.Queryable<BPA_Uint>().Where(it => it.IsDeleted == false && it.Id == x.StockUint).ToListAsync();
                                if (itemUint.Count > 0)
                                {
                                    itemUint.ForEach(x =>
                                    {
                                        if (!itemName.Contains(x.Name))
                                        {
                                            itemName += x.Name + "、";
                                        }
                                    });
                                }

                                x.itemName = itemName;
                            }
                        });
                    }
                }
            }
            else
            {
                //批量删除是判断是否可以删除
                if (items.Count > 0)
                {
                    items.ForEach(async x =>
                    {
                        if (typeValue == "1")
                        {
                            var itemCls = await DbClient.Queryable<BPA_ProductType>().Where(it => it.IsDeleted == false && it.Id == x.TypeID).ToListAsync();
                            if (itemCls.Count > 0)
                            {
                                itemCls.ForEach(x =>
                                {
                                    if (!itemName.Contains(x.Name))
                                    {
                                        itemName += x.Name + "、";
                                    }
                                });
                            }

                            x.itemName = itemName;
                        }
                        else
                        {
                            var itemUint = await DbClient.Queryable<BPA_Uint>().Where(it => it.IsDeleted == false && it.Id == x.StockUint).ToListAsync();
                            if (itemUint.Count > 0)
                            {
                                itemUint.ForEach(x =>
                                {
                                    if (!itemName.Contains(x.Name))
                                    {
                                        itemName += x.Name + "、";
                                    }
                                });
                            }

                            x.itemName = itemName;
                        }
                    });
                }
            }

            return BaseResult<List<ProductInfoDto>>.BuildOK(items);
        }

        return BaseResult<List<ProductInfoDto>>.BuildOK(new List<ProductInfoDto>());
    }
}