﻿using System;
using BPA.Common.Enums.CRM;
using BPA.Component.DTOCommon.BaseDTOs;
using BPA.Component.Extensions;
using BPA.SaaS.Stock.Api.DTO;
using BPA.SaaS.Stock.Api.IRepository.Product;
using BPA.SaaS.Stock.Api.IService.Product;

namespace BPA.SaaS.Stock.Api.Service.Product;

/// <summary>
/// 原料产品服务
/// </summary>
public class ProductService : IProductService
{
    private readonly Lazy<IProductRepository> _productRepository;

    /// <summary>
    /// 构造函数 
    /// </summary>
    /// <param name="productRepository"></param>
    public ProductService(Lazy<IProductRepository> productRepository)
    {
        _productRepository = productRepository;
    }

    /// <summary>
    /// 获取产品列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public Task<BaseResultPage<ProductListQuery>> GetProductList(ProductListQuery dto)
    {
        // TODO: 缺少获取GroupId
        return _productRepository.Value.GetProductList(dto, "ABC");
    }

    /// <summary>
    /// 查询物料信息
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public Task<BaseResultPage<ProductListByCodeQuery>> GetProductListBySrdId(ProductListDto dto) => _productRepository.Value.GetProductListBySrdId(dto);

    /// <summary>
    /// 添加产品
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public Task<BaseResult<bool>> AddProduct(ProductInfoDto dto)
    {
        if (dto == null) throw new ArgumentNullException(nameof(dto));
        if (dto.code.IsNotEmpty()) throw new ArgumentException(nameof(dto.code));
        if (dto.Id.IsNotEmpty()) throw new ArgumentException(nameof(dto.Id));
        if (dto.itemName.IsNotEmpty()) throw new ArgumentException(nameof(dto.itemName));
        //TODO:etc..
        return _productRepository.Value.AddProduct(dto);
    }

    /// <summary>
    /// 更新产品
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    public Task<BaseResult<bool>> UpdateProduct(ProductInfoDto dto)
    {
        if (dto == null) throw new ArgumentNullException(nameof(dto));
        if (dto.code.IsNotEmpty()) throw new ArgumentException(nameof(dto.code));
        if (dto.Id.IsNotEmpty()) throw new ArgumentException(nameof(dto.Id));
        if (dto.itemName.IsNotEmpty()) throw new ArgumentException(nameof(dto.itemName));
        //TODO:etc..
        return _productRepository.Value.UpdateProduct(dto);
    }

    /// <summary>
    /// 批量删除物料信息
    /// </summary>
    /// <param name="Ids"></param>
    /// <returns></returns>
    public Task<BaseResult<bool>> BatchDelProduct(List<string> Ids)
    {
        if (Ids == null) throw new ArgumentNullException(nameof(Ids));
        if (Ids.Count == 0) throw new ArgumentException("Value cannot be an empty collection.", nameof(Ids));
        //TODO:etc..
        return _productRepository.Value.BatchDelProduct(Ids);
    }

    /// <summary>
    /// 更新产品状态
    /// </summary>
    /// <param name="dto"></param>
    /// <exception cref="NotImplementedException"></exception>
    /// <returns></returns>
    [Obsolete("使用UpdateProduct方法")]
    public Task<bool> UpdateProductStatus(ProductInfoDto dto)
    {
        throw new NotImplementedException("使用UpdateProduct方法");
    }

    /// <summary>
    /// 根据物料类型、单位编号判断物料单位是否可以修改、删除或禁用
    /// </summary>
    /// <param name="ids">物料类型编号或者物料单位编号编号</param>
    /// <param name="typeValue">1:表示类型,2:单位</param>
    /// <param name="status">为空表示删除,不为空表示修改</param>
    /// <returns></returns>
    public Task<BaseResult<List<ProductInfoDto>>> IsModifyOrDel(string[] ids, string typeValue, EnumCommonStatus? status)
    {
        if (ids == null) throw new ArgumentNullException(nameof(ids));
        if (typeValue == null) throw new ArgumentNullException(nameof(typeValue));

        return _productRepository.Value.IsModifyOrDel(ids, typeValue, status);
    }

    /// <summary>
    /// 获取产品明细
    /// </summary>
    /// <param name="productID"></param>
    /// <returns></returns>
    public Task<BaseResult<List<ProductDetailedDto>>> GetProductCode(string productID)
    {
        if (productID == null) throw new ArgumentNullException(nameof(productID));
        return _productRepository.Value.GetProductCode(productID);
    }
}